#!/usr/bin/python
import os, sys, time, urllib, httplib, shutil, re
from urlparse import urlparse
from subprocess import Popen

def wiki_encode(name):
	""" encode the name for moniwiki """

	return name.encode('string_escape').replace('\\x', '_').replace(' ', '')

def guess_wiki_path(www_path = '/var/www'):
	""" guess the wiki path """

	moniwiki_path = os.path.join(www_path, 'moniwiki', 'data', 'text')

	if os.path.exists(moniwiki_path):
		return moniwiki_path

	# find moniwiki in document root for webserver
	for root, dirs, files in os.walk(www_path):
		if 'wiki.php' in files and 'wikilib.php' in files:
			return os.path.join(root, 'data', 'text')

	return '.'

def post_remote_wikipage(wiki_url, title, savetext, cookie = None):
	""" post a wikipage """

	headers = {"content-type": "application/x-www-form-urlencoded", "Accept": "text/plain", 'Cookie': cookie}
	parsed_url = urlparse(wiki_url + '/' + urllib.pathname2url(title))
	conn = httplib.HTTPConnection(parsed_url.netloc)
	params = urllib.urlencode({'action': 'savepage', 'savetext': savetext, 'datestamp': str(int(time.time()))})
	conn.request("POST", parsed_url.path, params, headers)
	conn.getresponse()

def nforge_login(nforge_url, username, password):
	""" login nFORGE """

	parsed_url = urlparse(nforge_url)
	conn = httplib.HTTPConnection(parsed_url.netloc)
	conn.request("GET", '/account/login.php')
	login_page = conn.getresponse().read()
	form_key = re.findall('name="form_key".*value="([^"]*)"', login_page)[0]

	params = urllib.urlencode({'login': '1', 'form_loginname': username, 'form_pw': password, 'form_key': form_key})
	headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
	conn.request("POST", '/account/login.php', params, headers)
	response = conn.getresponse()
	cookie = response.getheader('set-cookie')
	return cookie

def moniwiki_login(wiki_url, username, password):
	# login
	parsed_url = urlparse(wiki_url + '/UserPreferences')
	conn = httplib.HTTPConnection(parsed_url.netloc)
	params = urllib.urlencode({'login': 'Login', 'login_id': username, 'password': password, 'action': 'userform'})
	headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
	conn.request("POST", parsed_url.path, params, headers)
	response = conn.getresponse()

	return response.getheader('set-cookie')

def get_remote_wikipage(wiki_url, title, cookie = None):
	""" get a wikipage """

	parsed_url = urlparse(wiki_url + '/' + urllib.pathname2url(title))
	conn = httplib.HTTPConnection(parsed_url.netloc)
	headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain", 'Cookie': cookie}
	params = urllib.urlencode({'action': 'raw'})
	conn.request("GET", parsed_url.path + '?' + params, None, headers)
	response = conn.getresponse()

	return response.read()

def edit_remote_wikipage(editor, wiki_text_path, title, username, password, wiki_type, cookie = None):
	# login
	if not cookie:
		if wiki_type == 'nforge':
			cookie = nforge_login(wiki_text_path, username, password)
		else:
			cookie = moniwiki_login(wiki_text_path, username, password)

	# get it
	raw = get_remote_wikipage(wiki_text_path, title, cookie)

	file_name = '/tmp/' + title
	fp = open(file_name, 'w')
	fp.write(raw)
	fp.close()

	orig_file_name = file_name + '-orig'
	shutil.copy(file_name, orig_file_name)

	# edit it
	Popen([editor, file_name]).communicate()

	# post it
	if open(file_name, 'r').read() != open(orig_file_name, 'r').read():
		post_remote_wikipage(wiki_text_path, title, open(file_name, 'r').read(), cookie)

def wikifile_to_wikiname(filename):
	u = filename.replace('_', '\\x')
	exec 'x = unicode("' + u + '", "utf-8")'
	return x

def generate_link(target_path):
	files = os.listdir(target_path)

	for file in files:
		try:
			wikiname = wikifile_to_wikiname(file)
			dest = os.path.join(target_path, wikiname)
		except:
			continue
		
		if not os.path.exists(dest) and not '/' in wikiname:
			src = os.path.join(target_path, file)
			os.symlink(src, dest)
