#!/usr/bin/python
# -*- encoding: utf-8 -*-
import unittest
import shutil
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from mock_webserver import MoniWikiHandler
import time

# testing target
from wicilib.util import *

class TestWici(unittest.TestCase):
	def setUp(self):
		pass

	def testGuessWikiPath(self):
		# this directory is NOT a moniwiki directory
		# because wiki.php and wikilib.php does NOT exist
		shutil.rmtree('/tmp/wici-test', True)
		test_root = os.path.join('/tmp', 'wici-test')
		test_www_path = os.path.join(test_root, 'var', 'www')
		test_nimowiki_path = os.path.join(test_www_path, 'nimowiki')
		test_nimowiki_text_path = os.path.join(test_nimowiki_path, 'data', 'text')
		if not os.path.exists(test_nimowiki_text_path):
			os.makedirs(test_nimowiki_text_path)

		self.assertEquals('.', guess_wiki_path(test_www_path))

		# Now this is now a moniwiki directory
		# because wiki.php and wikilib.php exist
		fp = open(os.path.join(test_nimowiki_path, 'wiki.php'), 'w')
		fp.write('')
		fp.close()
		fp = open(os.path.join(test_nimowiki_path, 'wikilib.php'), 'w')
		fp.write('')
		fp.close()
		self.assertEquals(test_nimowiki_text_path, guess_wiki_path(test_www_path))

		# the directory is now a moniwiki directory
		test_moniwiki_text_path = os.path.join(test_www_path, 'moniwiki', 'data', 'text')
		if not os.path.exists(test_moniwiki_text_path):
			os.makedirs(test_moniwiki_text_path)

		self.assertEquals(test_moniwiki_text_path, guess_wiki_path(test_www_path))

	def testWikiEncode(self):
		self.assertEquals('_ea_b2_8c_ec_9e_84', wiki_encode('게임'))

	def testEditRemoteWikipage(self):
		os.system('python mock_webserver.py&')
		time.sleep(1)
		page = 'my frontpage'
		username = 'logintest'
		password = 'logintest'
		wiki_url = 'http://localhost:8800/wiki'
		title = 'FrontPage'

		cookie = moniwiki_login(wiki_url, username, password)
		post_remote_wikipage(wiki_url, title, page, cookie)
		raw = get_remote_wikipage(wiki_url, title, cookie)
  
		self.assertEquals(page, raw.split('\n')[0])

if __name__ == '__main__':
	unittest.main()
